<?php

//require_once 'Text/Diff/Renderer.php';

/**
 * "Inline2" diff renderer.
 *
 * Modified by steven whitbread to wrap everything in div tags, so that output can be formatted.
 *
 *
 * This class renders diffs in the Wiki-style "inline" format.
 *
 * $Horde: framework/Text_Diff/Diff/Renderer/inline.php,v 1.16 2006/01/08 00:06:57 jan Exp $
 *
 * @author  Ciprian Popovici
 * @package Text_Diff
 */
class Text_Diff_Renderer_inline2 extends Text_Diff_Renderer {

    /**
     * Number of leading context "lines" to preserve.
     */
    var $_leading_context_lines = 10000;

    /**
     * Number of trailing context "lines" to preserve.
     */
    var $_trailing_context_lines = 10000;

    /**
     * Prefix for inserted text.
     */
    var $_ins_prefix = '<ins>';

    /**
     * Suffix for inserted text.
     */
    var $_ins_suffix = '</ins>';

    /**
     * Prefix for deleted text.
     */
    var $_del_prefix = '<del>';

    /**
     * Suffix for deleted text.
     */
    var $_del_suffix = '</del>';

    /**
     * Header for each change block.
     */
    var $_block_header = '';

    /**
     * What are we currently splitting on? Used to recurse to show word-level
     * changes.
     */
    var $_split_level = 'lines';
    
    function _startDiff()
    {
        return '<div id="inlinediff">';
    }

    function _endDiff()
    {
        return '</div>';
    }

    function _blockHeader($xbeg, $xlen, $ybeg, $ylen)
    {
        return $this->_block_header;
    }

    function _startBlock($header)
    {
        return $header;
    }

    function _lines($lines, $prefix = ' ', $encode = true)
    {
        if ($encode) {
            array_walk($lines, array(&$this, '_encode'));
        }

        if ($this->_split_level == 'words') {
            return '<div>'.implode('', $lines).'</div>';
        } else {
            $fmtLines = Array();
            foreach($lines as $key => $line){
                $fmtLines[$key] = '<div>'.$line.'</div>';
            }
            return implode("\n", $fmtLines) . "\n";
        }
    }

    function _added($lines)
    {
        array_walk($lines, array(&$this, '_encode'));
        foreach($lines as $key => $line ){
            $lines[$key] = $this->_ins_prefix . $line . $this->_ins_suffix;
        }    
        //$lines[count($lines) - 1] .= $this->_ins_suffix;
        return $this->_lines($lines, ' ', false);
    }
    
    function _addedInline($lines)
    {
        array_walk($lines, array(&$this, '_encode'));
        $lines[0] = $this->_ins_prefix . $lines[0];
        $lines[count($lines) - 1] .= $this->_ins_suffix;
        return $this->_lines($lines, ' ', false);
    }

    function _deleted($lines, $words = false)
    {
        array_walk($lines, array(&$this, '_encode'));
        foreach($lines as $key => $line ){
            $lines[$key] = $this->_del_prefix . $line . $this->_del_suffix;
        }         
        return $this->_lines($lines, ' ', false);
    }

    function _changed($orig, $final)
    {
        /* If we've already split on words, don't try to do so again - just
         * display. */
        if ($this->_split_level == 'words') {
            $prefix = '';
            while ($orig[0] !== false && $final[0] !== false &&
                   substr($orig[0], 0, 1) == ' ' &&
                   substr($final[0], 0, 1) == ' ') {
                $prefix .= substr($orig[0], 0, 1);
                $orig[0] = substr($orig[0], 1);
                $final[0] = substr($final[0], 1);
            }
            return $prefix . $this->_deleted($orig) . $this->_added($final);
        }
        
        $html = '';
        
        /* These "if" statments catch conditions where the diff engine seems to return added lines as changes, 
         * this is seen by a difference between the count of the final array and the orig array. Also catch deleted lines (if there are any, not sure?).
         * it only seems to happen after the line with changes, not before.
        */  
        if(count($orig) == count($final)){
            for($i = 0; $i < count($orig); $i++){
                    /* Non-printing newline marker. */                    
                    $nl = "\0";
                    /* We want to split on word boundaries, but we need to
                     * preserve whitespace as well. Therefore we split on words,
                     * but include all blocks of whitespace in the wordlist. */
                    $diff = &new Text_Diff($this->_splitOnWords($orig[$i], $nl),
                                           $this->_splitOnWords($final[$i], $nl));            
                    /* Get the diff in inline format. */
                    $renderer = &new Text_Diff_Renderer_inline(array_merge($this->getParams(),
                                                                           array('split_level' => 'words')));            
                    /* Run the diff and get the output. */
                    $output = $renderer->render($diff);
            
                $html .= '<div>'.str_replace($nl, "\n", $renderer->render($diff)) .'</div>'. "\n";
            }
        }else{
            if(count($orig) < count($final)){
                for($i = 0; $i < count($orig); $i++){
                    /* Non-printing newline marker. */                    
                    $nl = "\0";
                    /* We want to split on word boundaries, but we need to
                     * preserve whitespace as well. Therefore we split on words,
                     * but include all blocks of whitespace in the wordlist. */
                    $diff = &new Text_Diff($this->_splitOnWords($orig[$i], $nl),
                                           $this->_splitOnWords($final[$i], $nl));            
                    /* Get the diff in inline format. */
                    $renderer = &new Text_Diff_Renderer_inline(array_merge($this->getParams(),
                                                                           array('split_level' => 'words')));            
                    /* Run the diff and get the output. */
                    $output = $renderer->render($diff);
                    $html .= '<div>'.str_replace($nl, "\n", $renderer->render($diff)) .'</div>'. "\n";
                }
                for($x = $i; $x < count($final); $x++){
                    $html .= '<div><ins>'.$final[$x].'</ins></div>'. "\n";
                }
            }
            if(count($orig) > count($final)){
                for($i = 0; $i < count($orig); $i++){
                    /* Non-printing newline marker. */                    
                    $nl = "\0";
                    /* We want to split on word boundaries, but we need to
                     * preserve whitespace as well. Therefore we split on words,
                     * but include all blocks of whitespace in the wordlist. */
                    $diff = &new Text_Diff($this->_splitOnWords($orig[$i], $nl),
                                           $this->_splitOnWords($final[$i], $nl));            
                    /* Get the diff in inline format. */
                    $renderer = &new Text_Diff_Renderer_inline(array_merge($this->getParams(),
                                                                           array('split_level' => 'words')));            
                    /* Run the diff and get the output. */
                    $output = $renderer->render($diff);
                    $html .= '<div>'.str_replace($nl, "\n", $renderer->render($diff)) .'</div>'. "\n";
                }
                for($x = $i; $x < count($final); $x++){
                    $html .= '<div><del>'.$orig[$x].'</del></div>'. "\n";
                }
            }
        }
        return $html;
    }
    
    function _context($lines)
    {
        return $this->_lines($lines). "\n";
    }

    function _splitOnWords($string, $newlineEscape = "\n")
    {
        $words = array();
        $length = strlen($string);
        $pos = 0;

        while ($pos < $length) {
            // Eat a word with any preceding whitespace.
            $spaces = strspn(substr($string, $pos), " \n");
            $nextpos = strcspn(substr($string, $pos + $spaces), " \n");
            $words[] = str_replace("\n", $newlineEscape, substr($string, $pos, $spaces + $nextpos));
            $pos += $spaces + $nextpos;
        }

        return $words;
    }

    function _encode(&$string)
    {
        $string = htmlspecialchars($string);
    }

}
